import os
import json

from em_tasks import logger
from em_tasks.spree.product_util import ProductUtil
from em_tasks.store.store_util import StoreUtil


class SpreeOutOfStockExporter():
  def __init__(self, store_code, output_dir):
    self.store = StoreUtil.get_store_by_code(store_code)
    if not self.store:
      return

    credential = json.loads(self.store.api_credential)
    self.product_util = ProductUtil(credential['endpoint'], credential['api_key'], credential.get('api_version', 'v1'))

    self.output_dir = os.path.abspath(os.path.expanduser(output_dir))
    if not os.path.isdir(self.output_dir):
      os.makedirs(self.output_dir)

  def run(self):
    sources = [
      {'source': 'Ebay_US', 'marketplace': 'us', 'method': 'get_ebay_products'},
      {'source': 'AMZ_US', 'marketplace': 'us', 'method': 'get_amz_products'},
      {'source': 'AMZ_UK', 'marketplace': 'uk', 'method': 'get_amz_products'},
      {'source': 'AMZ_DE', 'marketplace': 'de', 'method': 'get_amz_products'},
      {'source': 'AMZ_JP', 'marketplace': 'jp', 'method': 'get_amz_products'},
    ]
    for source in sources:
      in_stock_cnt = 0
      out_of_stock_cnt = 0
      prods = []
      for prod in getattr(self.product_util, source['method'])(source['marketplace']):
        if prod['in_stock'] == 'true':
          in_stock_cnt += 1
          continue

        prods.append(str(prod['product_id']))
        out_of_stock_cnt += 1

      output_path = os.path.join(self.output_dir, '{}_out_of_stock.txt'.format(source['source'].lower()))
      with open(output_path, 'w+', encoding='utf-8', errors='ignore') as fh:
        fh.write("\n".join(prods))

      logger.info('[SourceStats] Source: %s, InStock: %s, OutOfStock: %s', source['source'], in_stock_cnt, out_of_stock_cnt)

