import time

from dropshipping.utils.offer_converters import EsLowestOfferListingOfferConverter
from dropshipping.utils.offer_filters import LowestOfferListingOfferFilter
from dropshipping.utils.offer_services import EsLowestOfferListingOfferService
from dropshipping.utils.price_finders import OfferServiceLowestOfferListingPriceFinder

from em_tasks import logger


class AmzOfferService():
  def __init__(self, host, port, user, password, filter_cond):
    self.filter_cond = filter_cond
    self.filter_cond['provider_type'] = 'min'
    self.offer_filter = LowestOfferListingOfferFilter(self.filter_cond)
    self.offer_converter = EsLowestOfferListingOfferConverter()

    self.es_offer_service = EsLowestOfferListingOfferService(host, port, user, password)

    self.es_finder = OfferServiceLowestOfferListingPriceFinder(
      offer_service=self.es_offer_service, offer_filter=self.offer_filter,
      offer_converter=self.offer_converter)

  def get_offers(self, marketplace, asins, condition='new'):
    offers = False
    max_retries = 3
    while max_retries > 0:
      try:
        offers = self.es_finder.find_offer_for_asins(asins, country=marketplace, condition=condition)
        if not isinstance(offers, dict):
          logger.info('[OfferService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        break
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)

    return offers

  def analyze_offers(self, marketplace, asins, condition='new'):
    result = {asin:{} for asin in asins}
    offers = False
    raw_service_offers = dict()

    marketplace = marketplace.lower()
    max_retries = 3
    while max_retries > 0:
      try:
        service_offers_resp = self.es_offer_service.get_lowest_offer_listings_for_asin(
          marketplace, asins, condition)

        if not isinstance(service_offers_resp, dict):
          logger.info('[OfferService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        for item in service_offers_resp.get('hits', {}).get('hits', []):
          raw_service_offers[item['_id']] = {
            'asin': item['_source']['asin'],
            'time': item['_source']['time']
          }

        for asin in asins:
          raw_service_offer = raw_service_offers.get(asin, None)
          result[asin]['has_offers'] = bool(raw_service_offer)

        offers, _ = self.es_finder.get_offers(service_offers_resp)
        for asin in asins:
          offer = offers.get(asin, None)
          result[asin]['has_offer'] = bool(offer)
          result[asin]['offer'] = offer

        break
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)

    return result
